import logging
import os.path

import gobject
import pygst
pygst.require('0.10')
import gst

class Transcoder:
    """Audio transcoder from variable formats to streamable MP3s."""

    def __init__(self, fn_source, fn_sink):
        self.fn_source = os.path.abspath(fn_source)
        self.fn_sink = os.path.abspath(fn_sink)

        assert(os.path.exists(self.fn_source))
        assert(not os.path.exists(self.fn_sink))

        self.pipeline = self._setup_pipeline()

        bus = self.pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self._on_message)

        gobject.threads_init()
        self.loop = gobject.MainLoop()

    def _setup_pipeline(self):
        p_str = 'decodebin name=in ! audioconvert ! lame vbr=4 vbr-quality=6 ! id3v2mux name=out'
        p = gst.parse_launch(p_str)

        source = gst.element_factory_make('filesrc')
        source.set_property('location', self.fn_source)

        sink = gst.element_factory_make('filesink')
        sink.set_property('location', self.fn_sink)

        p.add(source, sink)

        gst.element_link_many(source, p.get_by_name('in'))
        gst.element_link_many(p.get_by_name('out'), sink)

        return p

    def _on_message(self, bus, message):
        t = message.type

        if t == gst.MESSAGE_EOS:
            logging.debug('%s transcoded to %s.', self.fn_source, self.fn_sink)
        elif t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            logging.error(err)
            logging.debug(debug)
        else:
            return

        self.pipeline.set_state(gst.STATE_NULL)
        self.loop.quit()

    def run(self):
        self.pipeline.set_state(gst.STATE_PLAYING)
        self.loop.run()



from __future__ import with_statement

import contextlib
import logging
import optparse
import os
import shutil
import urlparse
import webbrowser

from wsgiref.simple_server import make_server

import pkg_resources
import webob

import moxie.web

log = logging.getLogger(__name__)

def _setup_logging():
    _log_handler = logging.StreamHandler()
    _log_handler.setFormatter(logging.Formatter("%(message)s"))
    _log_handler.setLevel(logging.INFO)

    log.addHandler(_log_handler)

def local(bindaddr = 'localhost', port = 8080):
    """Deploy a test web server."""

    server = make_server(bindaddr, port, moxie.web.app())

    url = 'http://%s:%u/' % (bindaddr, port)

    print url
    webbrowser.open(url)

    server.serve_forever()

def static():
    """Deploy a set of static files to a directory."""

    # Parse the command-line.

    parser = optparse.OptionParser(usage='Usage: %prog [options] [directories ...]',
                                   description='moxie makes mixtapes!')
    parser.add_option('-f', '--force', help='overwrite existing files', action='store_true')
    parser.add_option('-v', '--verbose', help='explain what is being done', action='store_true')
    parser.add_option('-u', '--url', help='the directory\'s base URL', action='store')
    (options, args) = parser.parse_args()

    # Set logging verbosity.

    _setup_logging()

    if options.verbose:
        log.setLevel(logging.INFO)
    else:
        log.setLevel(logging.WARN)

    # Deploy to all specified directories. Use the current directory if none
    # were specified.

    if not args:
        args = ['.']
        log.info('Using current directory...')

    for d in args:
        app = moxie.web.app(d)

        if not app.music:
            log.error("Skipping %s (no music)" % d)
            continue

        # Generate the dynamic files.

        if options.url:
            options.url = options.url.strip('/')
        else:
            log.warn('No base URL specified (--url). Expect weirdness!')

        for uri, func in moxie.web.uri.uris(app):
            req = webob.Request.blank('/' + uri, base_url=options.url)
            res = req.get_response(app)

            fn = os.path.join(d, uri if uri else 'index.html')

            if os.path.exists(fn) and not options.force:
                log.warn("Skipping %s (file exists)" % fn)
            else:
                with file(fn, 'w') as f:
                    f.write(res.body)
                    log.info("Wrote %s" % fn)

        # Generate the static files.

        for bfn in pkg_resources.resource_listdir(__name__, 'static'):
            stream = pkg_resources.resource_stream(__name__, os.path.join('static', bfn))

            with contextlib.closing(stream) as f_in:
                fn = os.path.join(d, bfn)

                if os.path.exists(fn) and not options.force:
                    log.warn("Skipping %s (file exists)" % fn)
                else:
                    with file(fn, 'w') as f_out:
                        shutil.copyfileobj(f_in, f_out)
                        log.info("Wrote %s" % fn)



import glob
import os
import os.path

import mutagen
import mutagen.easyid3
import mutagen.id3

class TrackList(dict):
    """A dictionary with keys as MP3 files and values as TrackInfo instances."""

    HEADER = 'README'

    def __init__(self, directory):
        if not os.path.isdir(directory):
            raise IOError("%s not a directory" % directory)

        self.directory = directory

        try:
            header = [line.strip()
                      for line in file(os.path.join(directory, self.HEADER)).readlines()]
        except IOError:
            self.title = "A Moxie Mixtape!"
            self.subtitle = "Make a README"
        else:
            self.title = header[0]
            self.subtitle = '\n'.join(header[1:])

        for fn in glob.glob(os.path.join(directory, '*.mp3')):
            self[os.path.basename(fn)] = TrackInfo(fn)

class TrackInfo:
    """Metadata for audio files."""

    def __init__(self, filename):
        try:
            self._load(filename)
        except mutagen.id3.error:
            self.album = 'No Album'
            self.artist = 'No Artist'
            self.duration = '?:??'
            self.length = 0
            self.title = 'No Title'
            self.size = 0

    def _load(self, filename):
        short_tags = full_tags = mutagen.File(filename)

        if isinstance(full_tags, mutagen.mp3.MP3):
            short_tags = mutagen.easyid3.EasyID3(filename)

        self.album = short_tags.get('album', ['No Album'])[0]
        self.artist = short_tags.get('artist', ['No Artist'])[0]
        self.duration = "%u:%.2d" % (full_tags.info.length / 60, full_tags.info.length % 60)
        self.length = full_tags.info.length
        self.title = short_tags.get('title', ['No Title'])[0]
        self.size = os.stat(filename).st_size



import os.path

import mako.lookup
import selector
import static
import webob

import pkg_resources

import moxie.music

class uri(object):
    """A decorator for instance functions to attach a .uri_path and associate a template."""

    TEMPLATES = mako.lookup.TemplateLookup(directories = [pkg_resources.resource_filename(__name__, 'templates')],
                                           filesystem_checks = True)

    def __init__(self, path, template):
        self.uri_path = path
        self.template = template

    def __call__(self, func):
        def replacement(instance, environ, start_response):
            req = webob.Request(environ)
            tmpl = self.TEMPLATES.get_template(self.template)

            resp = func(instance, req, tmpl, **req.urlvars)

            if isinstance(resp, basestring):
                resp = webob.Response(body = resp)
            elif isinstance(resp, dict):
                resp = webob.Response(**resp)

            return resp(environ, start_response)

        replacement.uri_path = self.uri_path

        return replacement

    @classmethod
    def uris(cls, instance):
        """Return the (uri, function) pairs on an instance class."""

        for name in dir(instance):
            func = getattr(instance, name)

            if hasattr(func, 'uri_path'):
                yield func.uri_path, func

class app(selector.Selector):
    """WSGI application for Moxie."""

    def __init__(self, directory = '.'):
        selector.Selector.__init__(self, consume_path = False, prefix = '/')

        self.music = moxie.music.TrackList(directory)

        # Register the dynamic URIs.
        for path, func in uri.uris(self):
            self.add(path, GET = func)

        # Register the static URIs.
        static_app = static.Cling(pkg_resources.resource_filename(__name__, 'static'))

        for fn in pkg_resources.resource_listdir(__name__, 'static'):
            self.add(fn, GET = static_app)

        # Register the music.
        music_app = static.Cling(directory)

        for fn in self.music:
            self.add(fn, GET = music_app)

        # Special-case for user CSS customizations.
        if os.path.exists(os.path.join(directory, 'local.css')):
            self.add('local.css', GET = music_app)

    @uri('', 'index.html')
    def index(self, request, template):
        return template.render(tracklist = self.music)

    @uri('index.xspf', 'xspf.xml')
    def xspf(self, request, template):
        return {'content_type': 'application/xspf+xml',
                'body': template.render(tracklist = self.music)}
 
    @uri('index.rss', 'rss.xml')
    def rss(self, request, template):
        return {'content_type': 'application/rss+xml',
                'body': template.render(tracklist = self.music,
                                        request = request)}		



#!/usr/bin/env python

try:
    import pygst
    pygst.require('0.10')
except ImportError:
    logging.info('GStreamer bindings should be installed: http://pygstdocs.berlios.de/')

from setuptools import setup, find_packages

setup(
    name = 'moxie',
    version = '8.12.1',
    description = 'Makes mixtapes!',
    url = 'http://github.com/quad/moxie',

    author = 'Scott Robinson',
    author_email = 'scott@quadhome.com',

    maintainer = 'Moxie Hackers',
    maintainer_email = 'moxie-hackers@googlegroups.com',

    license = 'GPLv3',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Multimedia :: Sound/Audio :: Players :: MP3',
        'Topic :: Utilities',
    ],

    packages = ['moxie'],
    include_package_data = True,

    install_requires = [
        'flup', # BSD
        'Mako', # MIT
        'Markdown >= 1.7', # BSD
        'mutagen >= 1.15', # GPLv2
        'selector', # LGPLv2.1
        'static', # LGPLv2.1
        'WebOb', # MIT
    ],
    dependency_links = [
        # mutagen
        "http://code.google.com/p/quodlibet/downloads/list",
    ],

    entry_points = {
        'console_scripts': [
            'moxie-test = moxie.deploy:local',
            'moxie-static = moxie.deploy:static',
        ]
    },

    test_suite = 'nose.collector',
)
